<?php

declare(strict_types=1);

namespace Drupal\config_change_track\Drush\Commands;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\config_change_track\Tracker;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides drush commands for Config Change Track.
 */
final class ConfigChangeTrackCommands extends DrushCommands {

  /**
   * Constructs a ConfigChangeTrackCommands object.
   */
  public function __construct(
    private readonly ClassResolverInterface $classResolver,
    private readonly TimeInterface $time,
  ) {
    parent::__construct();
  }

  /**
   * Creates the command.
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('class_resolver'),
      $container->get('datetime.time'),
    );
  }

  /**
   * Displays 1 if config needs exporting, or 0 otherwise.
   */
  #[CLI\Command(name: 'config-change-track:needs-export')]
  public function needsExport(): void {
    $this->io()->writeln($this->getTracker()->needsExport() ? '1' : '0');
  }

  /**
   * Sets the time of the last successful config export.
   *
   * @param array{time: ?string} $options
   *   An associative array of options, keyed by:
   *   - time: the unix timestamp to set as the time of last successful export
   *       or NULL to use the current request time.
   */
  #[CLI\Command(name: 'config-change-track:set-last-export')]
  #[CLI\Option(name: 'time', description: 'The timestamp of the export')]
  public function setLastExport(array $options = ['time' => NULL]): void {
    $time = isset($options['time']) ? intval($options['time']) : $this->time->getRequestTime();
    $this->getTracker()->setLastExport($time);
  }

  /**
   * Returns the tracker object.
   */
  private function getTracker(): Tracker {
    return $this->classResolver->getInstanceFromDefinition(Tracker::class);
  }

}
