<?php

declare(strict_types=1);

namespace Drupal\config_change_track;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Records when config changes and informs when it's ready for export.
 *
 * @todo Strictly speaking it's possible for a user to continually update config
 *   at the right frequency to prevent it ever being marked as ready for export.
 *   We could close the loophole by having a maximum time between exports.
 */
final class Tracker implements ContainerInjectionInterface {

  /**
   * Creates the tracker.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   The config.
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreInterface $store
   *   The key value store service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    private readonly ImmutableConfig $config,
    private readonly KeyValueStoreInterface $store,
    private readonly TimeInterface $time,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('config.factory')->get('config_change_track.settings'),
      $container->get('keyvalue')->get('config_change_track'),
      $container->get('datetime.time')
    );
  }

  /**
   * Records when the latest config update occurred.
   *
   * @param int|null $time
   *   (optional) The time the update occurred, or NULL to use the request time.
   */
  public function setLatestUpdate(?int $time = NULL): void {
    if ($time === NULL) {
      $time = $this->time->getRequestTime();
    }
    $this->store->set('last_update', $time);
  }

  /**
   * Records when the latest config export occurred.
   *
   * @param int|null $time
   *   (optional) The time the export occurred, or NULL to use the request time.
   */
  public function setLastExport(?int $time = NULL): void {
    if ($time === NULL) {
      $time = $this->time->getRequestTime();
    }
    $this->store->set('last_export', $time);
  }

  /**
   * Checks if the config needs exporting.
   *
   * @param int|null $time
   *   (optional) The time to check, or NULL to use the request time.
   */
  public function needsExport(?int $time = NULL): bool {
    $defaults = [
      'last_update' => 1,
      'last_export' => 0,
    ];
    [
      'last_update' => $last_update,
      'last_export' => $last_export,
    ] = $this->store->getMultiple(array_keys($defaults)) + $defaults;
    if ($last_export >= $last_update) {
      return FALSE;
    }
    if ($time === NULL) {
      $time = $this->time->getRequestTime();
    }
    $export_delay = $this->config->get('export_delay');
    assert(is_int($export_delay));
    return $time > ($last_update + $export_delay);
  }

}
